@@ -55,10 +55,49 @@ class WeedOperation(object):
         self.master = WeedMaster(master_host, master_port)
 
 
-    def get_fid_full_url(self, fid):
+
+
+    def get_volume_fid_full_url(self, fid):
+        ''' return a random fid_full_url of volume by @fid
+
+        eg: (randomly choosed from locations)
+          return:  'http://127.0.0.1:27000/3,1234101234'  or
+          return:  'http://127.0.0.1:27001/3,1234101234'  or
+          return:  'http://127.0.0.1:27002/3,1234101234'
+        '''
+
+
+    def acquire_new_fids(self, count=1):
         """
-        get operatable full_url by fid.
-        lookup info returns:
+        get a new avalable fid or a fid list
+        return:
+          ['3,12301234'] or
+          ['3,12301234', '3,12301234_1', '3,12301234_2',  '3,12301234_3', etc..]
+
+        ----------
+
+        """
+        wak = self.master.acquire_new_assign_key(count=count)
+        # print(wak)
+        fid = wak.fid
+
+        if count == 1:
+            return [fid]
+        else:
+            fids = [fid] + [fid + ('_%d' % (i + 1)) for i in range(count)]
+            return fids
+
+
+
+    def get_fid_full_url(self, fid):
+        ''' return a random fid_full_url of volume by @fid
+
+        eg: (randomly choosed from locations)
+          return:  'http://127.0.0.1:27000/3,1234101234'  or
+          return:  'http://127.0.0.1:27001/3,1234101234'  or
+          return:  'http://127.0.0.1:27002/3,1234101234'
+
+        tip: WeedMaster.looup returns a dict like below:
         -----------
         {
           "locations": [
@@ -71,7 +110,7 @@ class WeedOperation(object):
         -----------
 
         return something like: 'http://127.0.0.1:8080/3,0230203913'
-        """
+        '''
         result = None
         volume_id = fid.split(',')[0]
         full_url = ''
@@ -102,35 +141,33 @@ class WeedOperation(object):
         return a WeedOperationResponse instance
         """
         LOGGER.info('|--> Putting file@fid:%s, fname:%s' % (fid, fname) )
-
         fid_full_url = 'wrong_url'
-        if not fid:
-            try:
+        _fid = fid
+        try:
+            if not fid:
                 wak = self.master.acquire_new_assign_key()
                 # print(wak)
+                _fid = wak.fid
+                LOGGER.debug('no fid. accquired new one: "%s"' % _fid)
                 fid_full_url = wak.fid_full_url
-                LOGGER.info('Putting file, Acquired new fid: %s, fid_full_url:%s for file: fp: %s, fname: %s'
-                             % (wak.fid, fid_full_url, fp, fname))
-                data = put_file(fp, fid_full_url, fname)
-                LOGGER.info('%s' % data)
-                data.fid = wak.fid
-                return data
-            except Exception as e:
-                err_msg = 'Could not put file. fp: "%s", fname: "%s", fid_full_url: "%s", e: %s' % (fp, fname, fid_full_url, e)
-                LOGGER.error(err_msg)
-                return None
-
-        else:
-            try:
+            else:
                 fid_full_url = self.get_fid_full_url(fid)
-                LOGGER.info('Putting file: fp: %s, fname: %s with fid: %s, fid_full_url: %s' % (fp, fname, fid, fid_full_url))
-                data = put_file(fp, fid_full_url, fname)
-                data.fid = fid
-                return data
-            except Exception as e:
-                err_msg = 'Could not put file with fid: %s, fid_full_url: %s, fp: %s, fname: %s, \nException is: %s' % (fid, fid_full_url, fp, fname, e)
-                LOGGER.error(err_msg)
-                return None
+        except Exception as e:
+            err_msg = 'Could not put file. fp: "%s", fname: "%s", fid_full_url: "%s", e: %s' % (fp, fname, fid_full_url, e)
+            LOGGER.error(err_msg)
+            return None
+
+        try:
+            LOGGER.info('Putting file with fid: %s, fid_full_url:%s for file: fp: %s, fname: %s'
+                         % (_fid, fid_full_url, fp, fname))
+            data = put_file(fp, fid_full_url, fname)
+            LOGGER.info('%s' % data)
+            data.fid = _fid
+            return data
+        except Exception as e:
+            err_msg = 'Could not put file. fp: "%s", fname: "%s", fid_full_url: "%s", e: %s' % (fp, fname, fid_full_url, e)
+            LOGGER.error(err_msg)
+            return None
 
 
     def get(self, fid, fname='', just_url=True):
